import os
import sys
sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

import pytest
from link_crab.session_manager import make_session

@pytest.fixture(scope='session')
def test_session():
    session = make_session()
    return session



import os
import sys
sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

from link_crab.session_manager import make_session
from link_crab.exercise_url import exercise_url
import pytest
from tests.test_app import mock_app
from tests.test_helpers import test_session

# These tests are highly dependent on the testapp.py
def test_url_validations(test_session):
    assert exercise_url(test_session,'Á://127.0.0.1:5000') == False
    assert exercise_url(test_session,'http://127.0.0.1:5000') != False

def test_url_status_codes(test_session):
    assert exercise_url(test_session,'http://127.0.0.1:5000')[1] == 200
    assert exercise_url(test_session,'http://127.0.0.1:5000/missing')[1] == 404

def test_accessibility(test_session):
    assert exercise_url(test_session,'http://127.0.0.1:5000/')[3] == True
    assert exercise_url(test_session,'http://127.0.0.1:5000/missing')[3] == False # Inaccessible because of status code is not ok
    assert exercise_url(test_session,'http://127.0.0.1:5000/member')[3] == False # Inaccessible because of permission


import os
import sys
sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

from link_crab.session_manager import make_session
from link_crab.exercise_url import exercise_url
from tests.test_app import app
import pytest
from tests.test_app import mock_app

@pytest.fixture(scope='session')
def test_session_mock():
    member_mockuser = {
        'login_url': 'http://127.0.0.1:5000/user/sign-in',
        'email': 'member@example.com',
        'password': 'Password1',
        'password_locator_id': 'password',
        'email_locator_id': 'email'
    }
    session = make_session(member_mockuser)
    return session

# These tests are highly dependent on the testapp.py
def test_access(test_session_mock):
    assert exercise_url(test_session_mock,'http://127.0.0.1:5000/members')[3] == True


import requests
from urllib.request import urljoin, urlparse
import logging

module_logger = logging.getLogger(__name__)

def exercise_url(session, url):
    '''
    Excersize url and gives back the following array:
    [0] visited url
    [1] status code
    [2] response time
    [3] accessible (bool)
    [4] returned url (after all redirects)
    args: 
        session: the session made by session_manager.py
        url: the url to be exercised
    '''
    if not check_url_validity(url):
        module_logger.warning(f"invalid link: {url}")
        return False
    try:
        resp = session.get(url) # maybe we should just use  allow_redirects=False for accessibility 
        accessible = resp.ok and resp.url == url
        outcome=[url,resp.status_code, int(resp.elapsed.total_seconds()*1000), accessible, resp.url] #should we change this to a dict? (depends on memory usage)
        module_logger.debug(f"[*] {outcome[0]} - {outcome[1]} - {outcome[4]} - {outcome[2]} ms - accessible: {outcome[3]}")

    except:
        outcome=[url, 'err', 'err', False, 'err']
    return outcome


def check_url_validity(url):
    parsed = urlparse(url)
    return bool(parsed.netloc) and bool(parsed.scheme)





from datetime import datetime

import os
delimiter=','
#TODO: some test-reports folder would be nice tbh, The -t argument should be used
def save_linkdb_to_csv(link_db, domain_name, starting_time):
    if not os.path.exists(f'reports/{domain_name}'):
        os.makedirs(f'reports/{domain_name}')
    with open(f"reports/{domain_name}/{domain_name}_{starting_time}_exercised_links.csv", "w") as f:
        print(f"url{delimiter}status_code{delimiter}resp_url(after_redirects){delimiter}response_time(ms){delimiter}accessible?", file=f)
        for link in link_db:
            print(f"{link[0]}{delimiter}{link[1]}{delimiter}{link[4]}{delimiter}{link[2]}{delimiter}{link[3]}", file=f)

def save_permdb_to_csv(perm_db, domain_name, user_email,starting_time):
    if not os.path.exists(f'reports/{domain_name}'):
        os.makedirs(f'reports/{domain_name}')
    with open(f"reports/{domain_name}/{domain_name}_{starting_time}_{user_email}_permission_check_result.csv", "w") as f:
        print(f"url{delimiter}status_code{delimiter}response_time(ms){delimiter}accessible?{delimiter}should_be_accessible?{delimiter}assert_accessibility", file=f)
        for link in perm_db:
            print(f"{link[0]}{delimiter}{link[1]}{delimiter}{link[4]}{delimiter}{link[3]}{delimiter}{link[5]}{delimiter}{link[6]}", file=f)

def save_links(links, domain_name):
    if not os.path.exists(f'reports/{domain_name}'):
        os.makedirs(f'reports/{domain_name}')
    with open(f"reports/{domain_name}/{domain_name}_links.txt", "w") as f:
        for link in links:
            print(f"{link}", file=f)


try:
    import reporting as reporting
except:
    pass
try:
    import link_crab.reporting as reporting
except:
    pass

import requests
from bs4 import BeautifulSoup
from urllib.request import urljoin, urlparse
import logging

crawled_links = set()
links_to_crawl = set()
total_urls_visited = 0

module_logger = logging.getLogger(__name__)

# FIXME: hangs if he first link is "out of domain" or unreachable?
def gather_links(session, links, checked_domain):
    for link in links:
        links_to_crawl.add(link)
    while len(links_to_crawl) > 0:
        module_logger.info(f"PROGRESS: {len(links)} links found, {len(crawled_links)} already crawled, {len(links_to_crawl)} left to crawl.")
        for link in links_to_crawl:
            crawl(session, link, links, checked_domain)
        reporting.save_links(links, checked_domain)

    return links



def crawl(session,url, links, checked_domain):
    global total_urls_visited
    global crawled_links
    global links_to_crawl
    module_logger.debug (f'{total_urls_visited} - currently checked page: {url}')

    if  url in crawled_links :
        module_logger.debug(f'    skip: already visited: {url}')
        return
    crawled_links.add(url)

    parsed = urlparse(url)
    if not (parsed.netloc == checked_domain):
        module_logger.debug(f'    skip: out of domain: {parsed.netloc}')
        return

    total_urls_visited += 1
    links = get_all_website_links(session, url, links)
    links_to_crawl= links-crawled_links


def get_all_website_links(session, url, links):
    """
    Returns URLs that is found on `url` in which belongs to the website
    """

    # domain name of the URL without the protocol
    domain_name = urlparse(url).netloc

    try:
        soup = BeautifulSoup(session.get(url).content, "html.parser")

        for a_tag in soup.findAll():
            href = a_tag.attrs.get("href")
            if href == "" or href is None:
                continue

            href = urljoin(url, href)
            parsed_href = urlparse(href)

            # remove URL GET parameters, URL fragments, etc.
            href = parsed_href.scheme + "://" + parsed_href.netloc + parsed_href.path
        
            if href in links:
               # print(f"[ ] link already gathered: {href}") # implement module_logger and logging level
               pass
            else:
                links.add(href)
               # print(f"[+] link found: {href}")
    except:
        module_logger.error(f"An exception occurred in getting website links from {url}")
    return links


import os
import sys
sys.path.append(os.path.join(os.path.dirname(__file__), '..'))
from tests.test_app import generate_mock_app
import time
import link_crab.session_manager
import link_crab.reporting
import link_crab.exercise_url
import link_crab.gather_links
import link_crab.csv_reader

import logging

from urllib.request import urlparse
import yaml
from datetime import datetime
import colorama

# Colorama init https://pypi.org/project/colorama/
colorama.init()
GREEN = colorama.Fore.GREEN
RED = colorama.Fore.RED
YELLOW = colorama.Fore.YELLOW
GRAY = colorama.Fore.LIGHTBLACK_EX
CYAN = colorama.Fore.CYAN
RESET = colorama.Fore.RESET

# CLI interface:
def main():
    import argparse
    parser = argparse.ArgumentParser(
        description="Link-Crab: A link crawler and permission testing tool, written in Python",
        usage=f"""
        Provide a path for your config yaml file. 
        Usable {CYAN}config keys{RESET}:
            {CYAN}starting_url: http://127.0.0.1:5000{RESET}
                Gather the reachable links in the starting_url's page and all of its subpages.
                After collecting all the links, the link exerciser load every in-domain url with a GET request, and measures 
                status code, response time, response url after all redirects, and accessibility based on status code and response url
            
            {CYAN}path_to_link_perms: testapp_member_access.csv{RESET}
                Test accessibility of provided links. The csv should have a link and a should-access column. 
                asserts the link accessibility equals to provided should-access.
                A link is accessible if the response status code<400, and after redirets the respone url equals the starting url
                (some framework give a 404 for unaccessible pages or redriects to sign_in page)

            {CYAN}user:
                email: member@example.com
                email_locator_id: email
                login_url: http://127.0.0.1:5000/user/sign-in
                password: Password1
                password_locator_id: password{RESET}
                    Login with the help of selenium webdriver (chromedriver). You need to provide the url of the login form, 
                    and the id's of the email (or username) and password fields.
        """
        )
    parser.add_argument("config_yaml_path", metavar='config_yaml_path', type=str,
                        help="Path to the config yaml file.")
    parser.add_argument("-t" ,"--test", action='store_true', help="wind up the default test application for testing")
    parser.add_argument("-v","--verbose", action='store_true', help="sets verbosity level for the cli output.")

    args = parser.parse_args()
    config_yaml_path = args.config_yaml_path

    # setup:
    starting_time = datetime.now()
    config = {}
    with open(config_yaml_path) as f:
        config = yaml.load(f, Loader=yaml.FullLoader)

    starting_url = None
    try:
        starting_url = config['starting_url']
    except:
        pass

    path_to_link_perms = None
    try:
        path_to_link_perms = config['path_to_link_perms']
    except:
        pass

    checked_domain=None
    if starting_url:
        checked_domain = urlparse(starting_url).netloc
    if path_to_link_perms:
        link_perms = csv_reader.read_link_perms(path_to_link_perms)
        checked_domain = urlparse(link_perms[0][0]).netloc
    
    #logger.filehandler
    if not os.path.exists(f'reports/{checked_domain}'):
        os.makedirs(f'reports/{checked_domain}')  

    logger = logging.getLogger(__name__)
    logger.setLevel(logging.DEBUG)
    fh = logging.FileHandler(f'reports/{checked_domain}/{checked_domain}_{starting_time}.log')
    fh.setLevel(logging.DEBUG)

    #lofgger.consolehandler
    ch = logging.StreamHandler()
    if args.verbose:
        ch.setLevel(logging.DEBUG)
        print("being Verbose!")
    else:
        ch.setLevel(logging.INFO)

    formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    fh.setFormatter(formatter)
    ch.setFormatter(formatter)

    logger.addHandler(fh)
    logger.addHandler(ch)

    logger.info('----------------setup---------------------')
    logger.info(f"checked_domain: {checked_domain}")
   
    if args.test:
        generate_mock_app()

    user = None
    try:
        user = config['user']
    except:
        pass
    session = session_manager.make_session(user)

    links = set()

    links.add(starting_url)

    exit_value = 0
    link_error_count = 0
    perm_error_count = 0 

        
    # gathering:
    if starting_url:
        logger.info('----------------gather links---------------------')
        
        links.add(starting_url)
        links = gather_links.gather_links(session, links, checked_domain)

    # exercising:
        logger.info('----------------Excercise links---------------------')
        link_db = []
        # remake the session, because crawling through the log-out link logs us out :D
        session = session_manager.make_session(user)
        for link in links:
            exercise_outcome = exercise_url.exercise_url(session, link)
            link_db.append(exercise_outcome)
            if exercise_outcome[1] == "err" or exercise_outcome[1] > 400:
                exit_value = 1
                link_error_count = link_error_count + 1

        reporting.save_linkdb_to_csv(link_db, checked_domain,starting_time)

        logger.debug('----------------REPORT: Excercised links---------------------')
        for link in link_db:
            logger.debug(f"[*] {link[0]} - {link[1]} - {link[4]} - {link[2]} ms - accessible: {link[3]}")

    # permission checking:

    
    if path_to_link_perms:
        logger.info('----------------permission checking---------------------')
        link_perm_db = []
        link_perms = csv_reader.read_link_perms(path_to_link_perms) # TODO: remove as duplicate
        checked_domain = urlparse(link_perms[0][0]).netloc # TODO: remove as duplicate
        session = session_manager.make_session(user)
        for link_perm in link_perms:
            outcome = (exercise_url.exercise_url(session, link_perm[0]))
            assertion_outcome = 'FAILED'
            if outcome[3] == link_perm[1]:
                assertion_outcome = 'PASSED'
            outcome = outcome + [link_perm[1]] + [assertion_outcome]
            link_perm_db.append(outcome)

        reporting.save_permdb_to_csv(link_perm_db, checked_domain, user['email'],starting_time)

        logger.debug('----------------REPORT: Checked permissions---------------------')
        for link in link_perm_db:
            if link[6] != 'PASSED':
                exit_value = 1
                perm_error_count = perm_error_count + 1
            logger.debug(f"[*] {link[0]} - {link[1]} - {link[4]} - {link[2]} ms - accessible: {link[3]} - should-be?: {link[5]} - assert: {link[6]}")

    # Exit:
    if starting_url:
        print(f"Gathered {len(link_db)} links, {link_error_count} of them broken.")
    if path_to_link_perms:
        print(f"Checked {len(link_perm_db)} links for permission, {perm_error_count} of them is not met the expectations.")
    print(f"exit value:{exit_value}")
    sys.exit(exit_value)






import csv
import logging

module_logger = logging.getLogger(__name__)

def read_link_perms(path_to_csv):
    with open(path_to_csv, 'r') as file:
        link_perms=[]
        csv_file = csv.DictReader(file)
        for row in csv_file:
            if row['should-access'] != '':
                should_access=None
                if row['should-access'].lower() in ('true', '1', 'yes'):
                    should_access=True
                elif row['should-access'].lower() in ('false','no', '0'):
                    should_access=False
                else:
                    raise 'invalid should-access value in perm-set! use true/false'
                link_perms.append([row['link'],should_access])
    module_logger.debug(link_perms)
    return link_perms


# This file contains an example Flask-User application.
# To keep the example simple, we are applying some unusual techniques:
# - Placing everything in one file
# - Using class-based configuration (instead of file-based configuration)
# - Using string-based templates (instead of file-based templates)

import datetime
from flask import Flask, request, render_template_string
from flask_babelex import Babel
from flask_sqlalchemy import SQLAlchemy
from flask_user import current_user, login_required, roles_required, UserManager, UserMixin

import threading
import time
import pytest

# Class-based application configuration
class ConfigClass(object):
    """ Flask application config """

    # Flask settings
    SECRET_KEY = 'This is an INSECURE secret!! DO NOT use this in production!!'

    # Flask-SQLAlchemy settings
    SQLALCHEMY_DATABASE_URI = 'sqlite:///basic_app.sqlite'    # File-based SQL database
    SQLALCHEMY_TRACK_MODIFICATIONS = False    # Avoids SQLAlchemy warning

    # Flask-Mail SMTP server settings
    MAIL_SERVER = 'smtp.gmail.com'
    MAIL_PORT = 465
    MAIL_USE_SSL = True
    MAIL_USE_TLS = False
    MAIL_USERNAME = 'email@example.com'
    MAIL_PASSWORD = 'password'
    MAIL_DEFAULT_SENDER = '"MyApp" <noreply@example.com>'

    # Flask-User settings
    USER_APP_NAME = "Flask-User Basic App"      # Shown in and email templates and page footers
    USER_ENABLE_EMAIL = True        # Enable email authentication
    USER_ENABLE_USERNAME = False    # Disable username authentication
    USER_EMAIL_SENDER_NAME = USER_APP_NAME
    USER_EMAIL_SENDER_EMAIL = "noreply@example.com"


def create_app():
    """ Flask application factory """
    
    # Create Flask app load app.config
    app = Flask(__name__)
    app.config.from_object(__name__+'.ConfigClass')

    # Initialize Flask-BabelEx
    babel = Babel(app)

    # Initialize Flask-SQLAlchemy
    db = SQLAlchemy(app)

    # Define the User data-model.
    # NB: Make sure to add flask_user UserMixin !!!
    class User(db.Model, UserMixin):
        __tablename__ = 'users'
        id = db.Column(db.Integer, primary_key=True)
        active = db.Column('is_active', db.Boolean(), nullable=False, server_default='1')

        # User authentication information. The collation='NOCASE' is required
        # to search case insensitively when USER_IFIND_MODE is 'nocase_collation'.
        email = db.Column(db.String(255, collation='NOCASE'), nullable=False, unique=True)
        email_confirmed_at = db.Column(db.DateTime())
        password = db.Column(db.String(255), nullable=False, server_default='')

        # User information
        first_name = db.Column(db.String(100, collation='NOCASE'), nullable=False, server_default='')
        last_name = db.Column(db.String(100, collation='NOCASE'), nullable=False, server_default='')

        # Define the relationship to Role via UserRoles
        roles = db.relationship('Role', secondary='user_roles')

    # Define the Role data-model
    class Role(db.Model):
        __tablename__ = 'roles'
        id = db.Column(db.Integer(), primary_key=True)
        name = db.Column(db.String(50), unique=True)

    # Define the UserRoles association table
    class UserRoles(db.Model):
        __tablename__ = 'user_roles'
        id = db.Column(db.Integer(), primary_key=True)
        user_id = db.Column(db.Integer(), db.ForeignKey('users.id', ondelete='CASCADE'))
        role_id = db.Column(db.Integer(), db.ForeignKey('roles.id', ondelete='CASCADE'))

    # Setup Flask-User and specify the User data-model
    user_manager = UserManager(app, db, User)

    # Create all database tables
    db.create_all()

    # Create 'member@example.com' user with no roles
    if not User.query.filter(User.email == 'member@example.com').first():
        user = User(
            email='member@example.com',
            email_confirmed_at=datetime.datetime.utcnow(),
            password=user_manager.hash_password('Password1'),
        )
        db.session.add(user)
        db.session.commit()

    # Create 'admin@example.com' user with 'Admin' and 'Agent' roles
    if not User.query.filter(User.email == 'admin@example.com').first():
        user = User(
            email='admin@example.com',
            email_confirmed_at=datetime.datetime.utcnow(),
            password=user_manager.hash_password('Password1'),
        )
        user.roles.append(Role(name='Admin'))
        user.roles.append(Role(name='Agent'))
        db.session.add(user)
        db.session.commit()

    # The Home page is accessible to anyone
    @app.route('/')
    def home_page():
        return render_template_string("""
                {% extends "flask_user_layout.html" %}
                {% block content %}
                    <h2>{%trans%}Home page{%endtrans%}</h2>
                    <p><a href={{ url_for('user.register') }}>{%trans%}Register{%endtrans%}</a></p>
                    <p><a href={{ url_for('user.login') }}>{%trans%}Sign in{%endtrans%}</a></p>
                    <p><a href={{ url_for('home_page') }}>{%trans%}Home Page{%endtrans%}</a> (accessible to anyone)</p>
                    <p><a href={{ url_for('member_page') }}>{%trans%}Member Page{%endtrans%}</a> (login_required: member@example.com / Password1)</p>
                    <p><a href={{ url_for('admin_page') }}>{%trans%}Admin Page{%endtrans%}</a> (role_required: admin@example.com / Password1')</p>
                    <p><a href={{ url_for('user.logout') }}>{%trans%}Sign out{%endtrans%}</a></p>
                    <p><a href=missing>{%trans%}missing{%endtrans%}</a></p>
                    Put here some cool malformed hrefs, and make good validations
                {% endblock %}
                """)

    # The Members page is only accessible to authenticated users
    @app.route('/members')
    @login_required    # Use of @login_required decorator
    def member_page():
        return render_template_string("""
                {% extends "flask_user_layout.html" %}
                {% block content %}
                    <h2>{%trans%}Members page{%endtrans%}</h2>
                    <p><a href={{ url_for('user.register') }}>{%trans%}Register{%endtrans%}</a></p>
                    <p><a href={{ url_for('user.login') }}>{%trans%}Sign in{%endtrans%}</a></p>
                    <p><a href={{ url_for('home_page') }}>{%trans%}Home Page{%endtrans%}</a> (accessible to anyone)</p>
                    <p><a href={{ url_for('member_page') }}>{%trans%}Member Page{%endtrans%}</a> (login_required: member@example.com / Password1)</p>
                    <p><a href={{ url_for('admin_page') }}>{%trans%}Admin Page{%endtrans%}</a> (role_required: admin@example.com / Password1')</p>
                    <p><a href={{ url_for('user.logout') }}>{%trans%}Sign out{%endtrans%}</a></p>
                    <p><a href=missing_member_only>{%trans%}Member only test{%endtrans%}</a></p>
                {% endblock %}
                """)

    # The Admin page requires an 'Admin' role.
    @app.route('/admin')
    @roles_required('Admin')    # Use of @roles_required decorator
    def admin_page():
        return render_template_string("""
                {% extends "flask_user_layout.html" %}
                {% block content %}
                    <h2>{%trans%}Admin Page{%endtrans%}</h2>
                    <p><a href={{ url_for('user.register') }}>{%trans%}Register{%endtrans%}</a></p>
                    <p><a href={{ url_for('user.login') }}>{%trans%}Sign in{%endtrans%}</a></p>
                    <p><a href={{ url_for('home_page') }}>{%trans%}Home Page{%endtrans%}</a> (accessible to anyone)</p>
                    <p><a href={{ url_for('member_page') }}>{%trans%}Member Page{%endtrans%}</a> (login_required: member@example.com / Password1)</p>
                    <p><a href={{ url_for('admin_page') }}>{%trans%}Admin Page{%endtrans%}</a> (role_required: admin@example.com / Password1')</p>
                    <p><a href={{ url_for('user.logout') }}>{%trans%}Sign out{%endtrans%}</a></p>
                {% endblock %}
                """)

    return app


app = create_app()


if __name__ == '__main__':
    import logging
    log = logging.getLogger('werkzeug')
    log.setLevel(logging.ERROR)
    app.run(host='127.0.0.1', port=5000, debug=True)



def flaskThread():
    import logging
    log = logging.getLogger('werkzeug')
    log.setLevel(logging.ERROR)
    app.run(host='127.0.0.1', port=5000, debug=False)

def generate_mock_app():
    """
    Quickly wind up a test app for development
    """
    thread = threading.Thread(target=flaskThread)
    thread.daemon = True
    thread.start()
    time.sleep(1)

@pytest.fixture(scope='session', autouse=True)
def mock_app():
    generate_mock_app()
    yield


import requests
from selenium import webdriver
import time
from selenium.webdriver.chrome.options import Options 
import logging

module_logger = logging.getLogger(__name__)

#FIXME: handle exception when webdriver is not present

def make_session(user=None):
    session = requests.session()
    if user:
        cookies = get_cookies_with_selenium(user)
        for cookie in cookies:
            cookie_obj = requests.cookies.create_cookie(domain=cookie['domain'],name=cookie['name'],value=cookie['value'])
            session.cookies.set_cookie(cookie_obj)
        module_logger.debug(session.cookies)
    return session

def get_cookies_with_selenium(user):
    login_url= user['login_url']
    email= user['email']
    email_locator_id = user['email_locator_id']
    password= user['password']
    password_locator_id = user['password_locator_id']

 

    chrome_options = Options()  
    chrome_options.add_argument("--headless")   

    driver = webdriver.Chrome(options=chrome_options)
    driver.get(login_url)
    
    
    email_field = driver.find_element_by_id(email_locator_id)
    
    email_field.clear()
    email_field.send_keys(email)

    password_field = driver.find_element_by_id(password_locator_id)
    password_field.clear()
    password_field.send_keys(password)

    submit_button = driver.find_element_by_xpath('//input[@type="submit"]')

    submit_button.click()
    # time.sleep(1)
    cookies = driver.get_cookies()
    module_logger.debug(f"got cookie with selenium: {cookies}")
    driver.close()
    return cookies


    


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="link-crab",
    version="0.2.1",
    author="Krisztián Pál Klucsik",
    author_email="klucsik.krisztian@gmail.com",
    description="A link crawler and permission testing tool for websites",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/klucsik/link-crab",
    packages=setuptools.find_packages(),
    install_requires=[
          'beautifulsoup4',
          'colorama',
          'Flask',
          'Flask-user',
          'Flask-babelex',
          'email_validator',
          'PyYAML',
          'requests',
          'selenium',
          'pytest'
      ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)


import os
import sys
sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

from link_crab.session_manager import make_session
from link_crab.gather_links import get_all_website_links, crawl, gather_links
import pytest
from tests.test_helpers import test_session
from tests.test_app import mock_app
import logging

def test_url_gathering(test_session):
    links = set()
    assert len(get_all_website_links(test_session,'http://127.0.0.1:5000', links)) == 8

def test_crawl_url_already_visited_not_collected(test_session, caplog):
    caplog.set_level(logging.DEBUG)
    links = set()
    url='http://www.example.com/xmpl'
    links.add(url)
    crawl(test_session,url,links,'www.example.com') # first run to collect url in crawled_links
    assert crawl(test_session,url,links,'www.example.com') == None
    captured = caplog.text
    assert "already visited" in captured

def test_crawl_url_out_of_domain_not_collected(test_session, caplog):
    caplog.set_level(logging.DEBUG)
    links = set()
    url='http://www.example.com/xmpl2'
    assert crawl(test_session,url,links,'www.not_correct_domain.com') == None
    captured = caplog.text
    assert "out of domain" in captured

def test_gather_links_hp(test_session):
    links = set()
    url='http://127.0.0.1:5000'
    links.add(url)

    outcome_links = {'http://127.0.0.1:5000', 'http://127.0.0.1:5000/', 'http://127.0.0.1:5000/admin', 'http://127.0.0.1:5000/members', 'http://127.0.0.1:5000/missing', 'http://127.0.0.1:5000/user/forgot-password', 'http://127.0.0.1:5000/user/missing', 'http://127.0.0.1:5000/user/register', 'http://127.0.0.1:5000/user/sign-in', 'http://127.0.0.1:5000/user/sign-out', 'http://netdna.bootstrapcdn.com/bootstrap/3.1.1/css/bootstrap.min.css'}
    assert  gather_links(test_session,links,'127.0.0.1:5000') == outcome_links